/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 */
#ifndef resources_h___
#define resources_h___

#define TIMER_1SECOND             40000
#define TIMER_5SECOND             40001
#define TIMER_10SECOND            40002

#define TIMER_1REPEAT             40003
#define TIMER_5REPEAT             40004
#define TIMER_10REPEAT            40005

#define TIMER_CANCEL              40006
#define TIMER_EXIT                40010

#endif /* resources_h___ */
