/* -*- Mode: IDL; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 */

#include "nsISupports.idl"
#include "nsIFileSpec.idl"

%{C++
#define NS_MSGPROTOCOLINFO_PROGID_PREFIX \
  "component://netscape/messenger/protocol/info;type="
%}


[scriptable, uuid(53d0214a-689c-11d3-80f5-00a0c900d445)]
interface nsIMsgProtocolInfo : nsISupports
{
    attribute nsIFileSpec defaultLocalPath;
    readonly attribute nsIIDPtr serverIID;
    readonly attribute boolean requiresUsername;
    readonly attribute boolean preflightPrettyNameWithEmailAddress;
    readonly attribute boolean canDelete;
    readonly attribute boolean canDuplicate;
	readonly attribute boolean defaultCopiesAndFoldersPrefsToServer;
};

