/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 */

/* stub interface for compatibility with XPIDL */
#include "nsISupports.idl"

%{C++
#include "nsString.h"
#include "nsIScriptContext.h"
%}
native nsString(nsString&);
native constnsString(const nsString&);

[uuid(be5c13bd-ba9f-11d2-96c4-0060b0fb9956)]
interface nsIDOMBaseAppCore : nsISupports
{
  void GetId(in nsString id);
  void Init(in constnsString id);
  void SetDocumentCharset(in constnsString charset);
};


%{C++
extern "C" NS_DOM nsresult NS_InitBaseAppCoreClass(nsIScriptContext *aContext, void **aPrototype);

extern "C" NS_DOM nsresult NS_NewScriptBaseAppCore(nsIScriptContext *aContext, nsISupports *aSupports, nsISupports *aParent, void **aReturn);
%}
