//{{NO_DEPENDENCIES}}
// Microsoft Developer Studio generated include file.
// Used by setuprsc.rc
//
#define IDS_ERROR_DIALOG_CREATE         1
#define IDS_ERROR_FAILED                2
#define IDS_ERROR_FILE_NOT_FOUND        3
#define IDS_ERROR_GET_SYSTEM_DIRECTORY_FAILED 4
#define IDS_ERROR_GET_WINDOWS_DIRECTORY_FAILED 5
#define IDS_DLGQUITTITLE                6
#define IDS_DLGQUITMSG                  7
#define IDS_DLG_REBOOT_TITLE            8
#define IDS_ERROR_GETPROCADDRESS        9
#define IDS_ERROR_WRITEPRIVATEPROFILESTRING 10
#define IDS_MSG_RETRIEVE_CONFIGINI      11
#define IDS_ERROR_CREATE_TEMP_DIR       12
#define IDS_DLGBROWSETITLE              13
#define IDS_ERROR_DETERMINING_DISK_SPACE 14
#define IDS_DLG_DISK_SPACE_CHECK_TITLE  15
#define IDS_DLG_DISK_SPACE_CHECK_SYS_MSG 16
#define IDS_DLG_DISK_SPACE_CHECK_CRUTIAL_MSG 16
#define IDS_DLG_DISK_SPACE_CHECK_MSG    17
#define IDS_ERROR_CREATE_DIRECTORY      18
#define IDS_STR_FILE_NUMBER             19
#define IDS_STR_FILENAME                20
#define IDS_MSG_SMARTUPDATE_START       21
#define IDS_MSG_CONFIGURING             22
#define IDS_ERROR_XPI_INSTALL           23
#define IDS_ERROR_SETUP_REQUIREMENT     24
#define IDS_DLG_EXTRACTING_TITLE        25
#define IDI_ICON1                       105
#define IDI_SETUP                       105
#define IDB_BITMAP_WELCOME              108
#define DLG_MESSAGE                     110
#define DLG_BROWSE_DIR                  503
#define IDC_STATUS                      1001
#define IDC_STATUS1                     1001
#define IDC_GAUGE                       1002
#define IDC_GAUGE_FILE                  1002
#define IDC_STATUS2                     1003
#define IDC_STATUS0                     1004
#define IDC_STATUS3                     1005
#define IDC_GAUGE_ARCHIVE               1006
#define IDC_EDIT_DESTINATION            1009
#define IDC_BUTTON_BROWSE               1010
#define IDC_EDIT_COMPONENT1             1013
#define IDC_EDIT_SUBCOMPONENT           1014
#define IDC_STATIC_SUBCOMPONENT         1016
#define IDC_CHECK1                      1017
#define IDC_STATIC_DESCRIPTION          1018
#define IDC_EDIT_PROGRAM_FOLDER         1019
#define IDC_CHECK2                      1023
#define IDC_LIST2                       1023
#define IDC_LIST                        1023
#define IDC_CHECK3                      1024
#define IDC_EDIT_LICENSE                1024
#define IDC_CHECK0                      1025
#define IDC_EDIT_CURRENT_SETTINGS       1026
#define IDC_LIST_COMPONENTS             1027
#define IDC_LIST_SUBCOMPONENTS          1029
#define IDC_STATIC_DRIVE_SPACE_REQUIRED 1030
#define IDC_STATIC_DRIVE_SPACE_AVAILABLE 1031
#define IDC_STATIC_DESTINATION          1032
#define IDC_STATIC0                     1033
#define IDC_STATIC1                     1034
#define IDC_STATIC2                     1035
#define IDC_MESSAGE1                    1040
#define IDC_PICT0                       1041
#define IDC_MESSAGE0                    1042
#define IDC_STATIC_ST0_DESCRIPTION      1042
#define IDC_STATIC_ST1_DESCRIPTION      1043
#define IDC_STATIC_ST2_DESCRIPTION      1044
#define IDC_STATIC_ST3_DESCRIPTION      1045
#define IDC_STATIC_MSG0                 1046
#define IDC_MESSAGE                     1049
#define IDC_LIST1                       1053
#define IDC_BUTTON1                     1055
#define IDC_README                      1055
#define DLG_WELCOME                     2001
#define DLG_LICENSE                     2002
#define DLG_SETUP_TYPE                  2003
#define DLG_SELECT_COMPONENTS_SINGLE    2004
#define DLG_SELECT_COMPONENTS           2004
#define DLG_SELECT_COMPONENTS_MULTI     2005
#define DLG_WINDOWS_INTEGRATION         2006
#define DLG_PROGRAM_FOLDER              2007
#define DLG_START_INSTALL               2008
#define DLG_EXTRACTING                  2009
#define DLG_RESTART                     10206
#define IDB_BOX_CHECKED                 10304
#define IDB_BOX_UNCHECKED               10306
#define IDC_RADIO_TYPICAL               11007
#define IDC_RADIO_ST0                   11007
#define IDC_RADIO_CUSTOM                11008
#define IDC_RADIO_ST2                   11008
#define IDC_RADIO_COMPACT               11009
#define IDC_RADIO_ST1                   11009
#define IDC_RADIO_ST3                   11010
#define IDC_RADIO_YES                   11011
#define IDC_RADIO_NO                    11012

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NO_MFC                     1
#define _APS_NEXT_RESOURCE_VALUE        112
#define _APS_NEXT_COMMAND_VALUE         40001
#define _APS_NEXT_CONTROL_VALUE         1058
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
